# Copyright (C) Jan 2020 Mellanox Technologies Ltd. All rights reserved.
# Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
#
# This software is available to you under a choice of one of two
# licenses.  You may choose to be licensed under the terms of the GNU
# General Public License (GPL) Version 2, available from the file
# COPYING in the main directory of this source tree, or the
# OpenIB.org BSD license below:
#
#     Redistribution and use in source and binary forms, with or
#     without modification, are permitted provided that the following
#     conditions are met:
#
#      - Redistributions of source code must retain the above
#        copyright notice, this list of conditions and the following
#        disclaimer.
#
#      - Redistributions in binary form must reproduce the above
#        copyright notice, this list of conditions and the following
#        disclaimer in the documentation and/or other materials
#        provided with the distribution.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# --


#######################################################
#
# SegmentFactory.py
# Python implementation of the Class SegmentFactory
# Generated by Enterprise Architect
# Created on:      14-Aug-2019 10:11:59 AM
# Original author: talve
#
#######################################################


class SegmentFactory:
    """This class is responsible for creating and registering the segments.
    """
    segments = {}

    @classmethod
    def register(cls, segment_type, obj):
        """This method should  be use from the specific segment in order to perform the
        segment registration.
        """
        cls.segments.update({segment_type: obj})

    @classmethod
    def create(cls, segment_type, *args):
        """create the segment and return the segment obj.
        """
        if segment_type not in cls.segments:
            raise Exception("segment type:{0} is not supported".format(hex(segment_type)))
        return cls.segments[segment_type](*args)
